#########################################################################
# Node-splitting FIXED EFFECTS MODEL EXAMPLE
# R script to run node-split for the MTC FE model using WinBUGS
#
# Uses R2WinBUGS package
#
# Thrombolytics Example
#  1. Need to include in the working directory the following files:
#             ThromboData.txt --- text file with data
#             fe node-splitR2.txt --- text file holding WinBUGS code
#
#   2. Output files will be
#             coda1.txt  --- holds coda output
#             codaIndex.txt --- holds indexes to coda output
#             data.txt --- holds all data as used by WinBUGS
#             log.odc and log.txt --- hold WinBUGS output
#             inits1.txt --- holds initial values as read by WinBUGS
#             script.txt --- WinBUGS script file with all commands to execute
#
#   3. Output files for each node should be transferred to a new directory 
#      as they will be overwritten in each new run
#
#   4. You  may need to edit the WinBUGS directory 'bd'
#
#   5. You will need to edit the working directory 'pathname' 
#      to suit your computer settings
#
#   6. Run script file
# 
#   7. To repeat for other node-splits need to change variable 'pair' 
#      and edit output file names
#
#########################################################################
#
# Declare the directory where WinBUGS is found in this computer
bd <- "C:/Program Files/WinBUGS14/"
#
# Declare working directory
pathname <- "C:/Documents and Settings/***userID***/My Documents/***etc***"
setwd(pathname)
#
# load package to call WinBUGS
library(R2WinBUGS)
#
# LOAD DATA MANIPULATING FUNCTIONS:
#
PairXY <- function(treat, pair)
# Check if pair(X,Y) in row i of data 
# and give baseline for data row i
{
  N <- nrow(treat)
  out <- cbind(split=rep(0,N), b=rep(0,N))
  for (i in 1:N) {
# returns positions of matches to elements of pair in t[i,]
# or zero if not present
   pos <- match(pair, treat[i,], nomatch=0)   # lenght = length(pair) = 2
   out[i,1] <- ifelse(prod(pos)>0, 1, 0)      # 1 if pair in line i, 0 o.w.
   out[i,2] <- ifelse(prod(pos)==0, 1, pos[1])
 }
out
}
#
NonbaseSweep <- function(index, na)
# gives na-1 indexes to sweep non-baseline arms only
{
  N <- NROW(na)
  C <- max(na)
  out <- matrix(nrow=N, ncol=C)
  for (i in 1:N) {
   for (k in 2:na[i]) {
     out[i,k] <- k - (index[i,"b"] >= k)
   }
  }
out
}
#
Sweeptreat <- function(treat, m)
# Builds matrix with non-baseline treatments
{
  N <- NROW(treat)
  C <- NCOL(m)
  out <- matrix(nrow=N, ncol=C)
  for (i in 1:N) {
   for (k in 2:C) {
     out[i,k] <- treat[i,m[i,k]]
   }
  }
out
}
#
Basetreat <- function(treat, b)
# Builds vector with baseline treatments
{
  N <- nrow(treat)
  out <- rep(0,N)
  for (i in 1:N) {
   out[i] <- treat[i,b[i]]
  }
out
}
#
# Setup subdirectories to hold results for each of 16 node-splits
dir.create("FEnode12")
#dir.create("FEnode13")
#dir.create("FEnode14")
# ...etc...
#
#
###########################################################################
# load data for MTC
MTCData <- read.table("ThromboData.txt", header=TRUE)
r <- data.matrix(MTCData[,c("r1", "r2", "r3")])
n <- data.matrix(MTCData[,c("n1", "n2", "n3")])
t <- data.matrix(MTCData[,c("t1", "t2", "t3")])
na <- data.matrix(MTCData[,"na"])
nt <- max(t, na.rm=TRUE)
ns <- nrow(r)
#
# define initial values
initv1 <- list(direct=0,  d=c(NA,0,0,0,0,0,0,0,0), mu=rep(0,ns))
#
############################################################################
#  NODE-SPLITTING ROUTINE
############################################################################
#
#
# Define node to split: (1,2)
pair <- c(1,2)
#
# BUILD EXTRA INPUT VARIABLES
# Calculate split (1 if node to split is present) and b (baseline position)
checkPair <- PairXY(t, pair)
# Build vector bi[i] with baseline treatment: t[i, b[i]]
bi <- Basetreat(t, checkPair[,"b"])
# Indexes to sweep non-baseline arms only
m <- NonbaseSweep(checkPair, na)
# Build matrix si[i,k] with non-baseline treatments: t[i, m[i,k]]
si <- Sweeptreat(t,m)
#
# Build data file: stored in the working directory as "data.txt"
bugs.data(list("r"=r,"n"=n,"t"=t,
               "na" = na[,1], "nt" = nt, "ns" = ns,
               "split" = checkPair[,"split"], "m" =m,
               "bi" = bi, "si" = si, "pair" = pair    ) )
# Call WinBUGS
# (ignore error in parameter "lor" it will still be saved to log.txt and coda1.txt)
#
bugs(data = "data.txt",
     inits = list(initv1),
     parameters.to.save = c("direct", "d", "prob","totresdev","lor", "diff"),
     model.file = "fe node-splitR2.txt",
     n.chains = 1,
     n.iter = 150000,
     n.burnin = 50000,
     n.thin = 1,
     bugs.directory = bd,
     working.directory=pathname,
     debug = FALSE )
#
# Import coda output
coda12 <- read.bugs("coda1.txt")
# Copy input and output files to relevant directory
file.copy("data.txt", "FEnode12/data.txt", overwrite=TRUE)
file.copy("log.odc", "FEnode12/log.odc", overwrite=TRUE)
file.copy("inits1.txt", "FEnode12/inits1.txt", overwrite=TRUE)
file.copy("script.txt", "FEnode12/script.txt", overwrite=TRUE)
#
# REPEAT FOR ALL OTHER NODES
# ...